//	AlbumCoverDisplay.cpp - Displays art for the current song.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AlbumCoverDisplay.h"

//------------------------------------------------------------------------------
AlbumCoverDisplay::AlbumCoverDisplay():
Component(),
image(0),
big(false),
bigIndex(0.0f),
origX(0),
origY(0),
origSize(56),
bigX(0),
bigY(0),
bigSize(100)
{
	
}

//------------------------------------------------------------------------------
AlbumCoverDisplay::~AlbumCoverDisplay()
{
	deleteAllChildren();
	delete image;
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay::paint(Graphics& g)
{
	g.setColour(Colours::white);
	g.fillAll();

	if(image)
	{
		//This makes no difference speed-wise anyway.
		//g.setImageResamplingQuality(Graphics::lowResamplingQuality);
		g.drawImage(image,
					1,
					1,
					(getWidth()-2),
					(getHeight()-2),
					0,
					0,
					image->getWidth(),
					image->getHeight());
	}

	g.setColour(Colours::black.withAlpha(0.5f));
	g.drawRect(0, 0, getWidth(), getHeight());
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay::mouseUp(const MouseEvent& e)
{
	if(!big && (bigIndex == 0.0f))
	{
		big = true;
		bigSize = getParentComponent()->getHeight();
		bigX = (getParentComponent()->getWidth()-bigSize)/2;
		bigY = 0;
		origX = getX();
		origY = getY();
		origSize = getHeight();
		startTimer(30);
	}
	else if(big && (bigIndex == 1.0f))
	{
		big = false;
		startTimer(30);
	}
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay::timerCallback()
{
	if(big)
	{
		float newX, newY, newSize;

		bigIndex += 0.05f;
		if(bigIndex >= 1.0f)
		{
			bigIndex = 1.0f;
			stopTimer();
		}

		newX = (origX * (1.0f-bigIndex)) + (bigX * bigIndex);
		newY = (origY * (1.0f-bigIndex)) + (bigY * bigIndex);
		newSize = (origSize * (1.0f-bigIndex)) + (bigSize * bigIndex);

		setBounds((int)newX, (int)newY, (int)newSize, (int)newSize);
		repaint();
	}
	else if(!big)
	{
		float newX, newY, newSize;

		bigIndex -= 0.05f;
		if(bigIndex <= 0.0f)
		{
			bigIndex = 0.0f;
			stopTimer();
		}

		newX = (origX * (1.0f-bigIndex)) + (bigX * bigIndex);
		newY = (origY * (1.0f-bigIndex)) + (bigY * bigIndex);
		newSize = (origSize * (1.0f-bigIndex)) + (bigSize * bigIndex);

		setBounds((int)newX, (int)newY, (int)newSize, (int)newSize);
		repaint();
	}
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay::loadImage(const String& filename)
{
	File phil(filename);

	if(image)
	{
		delete image;
		image = 0;
	}

	if(filename == T(""))
		return;

	//First look for <filename - extension>.jpg.
	{
		File phil2 = phil.withFileExtension(T("jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}

	//Next look for <filename - extension>.png.
	{
		File phil2 = phil.withFileExtension(T("png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}

	//If we get here, look for Folder.jpg.
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("Folder.jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}

	//folder.jpg.
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("folder.jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}

	//Folder.png.
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("Folder.png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}

	//folder.png.
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("folder.png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
			return;
		}
	}
}
