//	AlbumCoverDisplay2.cpp - Displays art for the current song using OpenGL.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "AlbumCoverDisplay2.h"

//The gl.h header from my crappy old version of MSVC only defines GL_BGR_EXT.
#ifndef GL_BGR
#define GL_BGR GL_BGR_EXT
#endif

//------------------------------------------------------------------------------
AlbumCoverDisplay2::AlbumCoverDisplay2():
OpenGLComponent(),
image(0),
big(false),
bigIndex(0.0f),
origX(0),
origY(0),
origSize(56),
bigX(0),
bigY(0),
bigSize(100),
loadTexture(false),
resized(false)
{
	
}

//------------------------------------------------------------------------------
AlbumCoverDisplay2::~AlbumCoverDisplay2()
{
	deleteAllChildren();
	delete image;
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay2::renderOpenGL()
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	if(loadTexture)
	{
		int line, pixel;
		Image *temp = image->createCopy(512, 512);
		const uint8* pixelData = temp->lockPixelDataReadOnly(0,
															 0,
															 512,
															 512,
															 line,
															 pixel);

		if(pixelData)
		{
			glBindTexture(GL_TEXTURE_2D, texture);
			glTexImage2D(GL_TEXTURE_2D,
						 0,
						 4,
						 512,
						 512,
						 0,
						 GL_BGR,
						 GL_UNSIGNED_BYTE,
						 pixelData);
			if(glGetError() == GL_NO_ERROR)
			{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			}

			temp->releasePixelDataReadOnly(pixelData);
		}
		delete temp;

		loadTexture = false;
	}

	if(resized)
	{
		glViewport(0, 0, getWidth(), getHeight());

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		glOrtho(0.0f, (float)getWidth(), (float)getHeight(), 0.0f, -1.0f, 1.0f);

		resized = false;
	}

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	if(image)
	{
		glBindTexture(GL_TEXTURE_2D, texture);
		glBegin(GL_QUADS);
		{
			glTexCoord2f(0, 0);
			glVertex2d(1, 1);

			glTexCoord2f(1, 0);
			glVertex2d(getWidth()-1, 1);

			glTexCoord2f(1, 1);
			glVertex2d(getWidth()-1, getHeight()-1);

			glTexCoord2f(0, 1);
			glVertex2d(1, getHeight()-1);
		}
		glEnd();
		glBindTexture(GL_TEXTURE_2D, 0);
	}
	else
	{
		glColor3f(1.0f, 1.0f, 1.0f);
		glBegin(GL_QUADS);
		{
			glVertex2d(1, 1);
			glVertex2d(getWidth()-1, 1);
			glVertex2d(getWidth()-1, getHeight()-1);
			glVertex2d(1, getHeight()-1);
		}
		glEnd();
	}
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay2::newOpenGLContextCreated()
{
	glViewport(0, 0, getWidth(), getHeight());

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0.0f, (float)getWidth(), (float)getHeight(), 0.0f, -1.0f, 1.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glClearColor(0.5f, 0.5f, 0.5f, 0.5f);

	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &texture);
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay2::mouseUp(const MouseEvent& e)
{
	if(!big && (bigIndex == 0.0f))
	{
		big = true;
		bigSize = getParentComponent()->getHeight();
		bigX = (getParentComponent()->getWidth()-bigSize)/2;
		bigY = 0;
		origX = getX();
		origY = getY();
		origSize = getHeight();
		startTimer(30);
	}
	else if(big && (bigIndex == 1.0f))
	{
		big = false;
		startTimer(30);
	}
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay2::timerCallback()
{
	if(big)
	{
		float newX, newY, newSize;

		bigIndex += 0.05f;
		if(bigIndex >= 1.0f)
		{
			bigIndex = 1.0f;
			stopTimer();
		}

		newX = (origX * (1.0f-bigIndex)) + (bigX * bigIndex);
		newY = (origY * (1.0f-bigIndex)) + (bigY * bigIndex);
		newSize = (origSize * (1.0f-bigIndex)) + (bigSize * bigIndex);

		setBounds((int)newX, (int)newY, (int)newSize, (int)newSize);
		resized = true;
		repaint();
	}
	else if(!big)
	{
		float newX, newY, newSize;

		bigIndex -= 0.05f;
		if(bigIndex <= 0.0f)
		{
			bigIndex = 0.0f;
			stopTimer();
		}

		newX = (origX * (1.0f-bigIndex)) + (bigX * bigIndex);
		newY = (origY * (1.0f-bigIndex)) + (bigY * bigIndex);
		newSize = (origSize * (1.0f-bigIndex)) + (bigSize * bigIndex);

		setBounds((int)newX, (int)newY, (int)newSize, (int)newSize);
		resized = true;
		repaint();
	}
}

//------------------------------------------------------------------------------
void AlbumCoverDisplay2::loadImage(const String& filename)
{
	File phil(filename);

	if(image)
	{
		delete image;
		image = 0;
		loadTexture = false;
	}

	if(filename == T(""))
		return;

	//First look for <filename - extension>.jpg.
	{
		File phil2 = phil.withFileExtension(T("jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	//Next look for <filename - extension>.png.
	if(!image)
	{
		File phil2 = phil.withFileExtension(T("png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	//If we get here, look for Folder.jpg.
	if(!image)
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("Folder.jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	//folder.jpg.
	if(!image)
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("folder.jpg"));

		if(phil2.existsAsFile())
		{
			JPEGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	//Folder.png.
	if(!image)
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("Folder.png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	//folder.png.
	if(!image)
	{
		File phil2 = phil.getParentDirectory().getChildFile(T("folder.png"));

		if(phil2.existsAsFile())
		{
			PNGImageFormat imageFormat;
			FileInputStream input(phil2);

			image = imageFormat.decodeImage(input);
		}
	}

	if(image)
		loadTexture = true;
}
