//	AlbumCoverDisplay2.h - Displays art for the current song using OpenGL.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef ALBUMCOVERDISPLAY2_H_
#define ALBUMCOVERDISPLAY2_H_

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>

#include "juce.h"

///	Displays art for the current song using OpenGL.
/*!
	Juce's drawing isn't really up to the task of resizing an image to take
	up the whole window (or at least, animating it), but OpenGL's great at that
	kind of thing.
 */
class AlbumCoverDisplay2 : public OpenGLComponent,
						   public Timer
{
  public:
	///	Constructor.
	AlbumCoverDisplay2();
	///	Destructor.
	~AlbumCoverDisplay2();

	///	Draws the album art.
	void renderOpenGL();
	///	Does some setup.
	void newOpenGLContextCreated();

	///	Used to resize the image to the full window.
	void mouseUp(const MouseEvent& e);
	///	Used to provide a fancy animation for resizing the image.
	void timerCallback();

	///	Loads a new image, derived from the filename passed in.
	void loadImage(const String& filename);

	///	Returns whether or not we're in 'big mode'.
	bool isBig() const {return big;};
  private:
	///	The image to display (original copy).
	Image *image;

	///	True when the image is resized to fit the entire window.
	bool big;
	///	Interpolation index when resizing.
	float bigIndex;
	///	Original x position.
	int origX;
	///	Original y position.
	int origY;
	///	Original size.
	int origSize;
	///	X position to move the image to when resized.
	int bigX;
	///	Y position to move the image to when resized.
	int bigY;
	///	Resized width/height.
	int bigSize;

	///	Our OpenGL texture.
	GLuint texture;
	///	Whether we need to load a new texture.
	bool loadTexture;
	///	Whether we need to re-do the viewport etc.
	bool resized;
};

#endif
