//	FolderList.h - Visual list of available folders.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef FOLDERLIST_H_
#define FOLDERLIST_H_

#include "juce.h"
#include <vector>

class MainWindow;

///	Visual list of queued songs.
class FolderList : public Component,
				   public ListBoxModel,
				   public DragAndDropTarget
{
  public:
	///	Constructor.
	FolderList(MainWindow *win);
	///	Destructor.
	~FolderList();

	///	Sizes songs.
	void resized();

	///	Used to load songs into the app.
	bool filesDropped(const StringArray& filenames, int mouseX, int mouseY);

	///	Returns the current number of rows.
	int getNumRows() {return folders.size()+1;};
	///	Draws the selected row.
	void paintListBoxItem(int rowNumber,
						  Graphics& g,
						  int width,
						  int height,
						  bool rowIsSelected);
	///	Used to update MainWindow's SongList when we select a new folder.
	void selectedRowsChanged(int lastRowSelected);
	///	Used to remove folders.
	void deleteKeyPressed(int lastRowSelected);

	///	For determining whether we're interested in a dragged thing.
	bool isInterestedInDragSource(const String& sourceDescription);
	///	Called when something's dropped on us.
	void itemDropped(const String& sourceDescription,
					 Component *sourceComponent,
					 int x,
					 int y);
	///	Called when something's dragged over us.
	void itemDragEnter(const String& sourceDescription,
					   Component *sourceComponent,
					   int x,
					   int y);
	///	Called when something's dragged over us.
	void itemDragExit(const String& sourceDescription,
					  Component *sourceComponent);

	///	Returns the selected folder path.
	const String& getSelectedFolder() const;
	///	Returns a vector of all the folders we keep track of.
	const std::vector<String>& getFolders() const {return folders;};

	juce_UseDebuggingNewOperator
  private:
	///	The ListBox listing all the folders.
	ListBox *listBox;

	///	Holds the folders we display.
	std::vector<String> folders;

	///	A pointer to the main window.
	MainWindow *mainWindow;

	///	The currently-selected row.
	int selectedRow;

	///	True when something's being dragged over us.
	bool somethingDragged;
};

#endif
