//	LAMEAudioFormat.cpp - LAME-based audio format for reading MP3s.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "LAMEAudioFormat.h"

#include "lame.h"

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
#define formatName                          TRANS("MP3 file")
static const tchar* const extensions[] =    { T(".mp3"), 0 };

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class LAMEReader : public AudioFormatReader
{
  public:
	//--------------------------------------------------------------------------
	LAMEReader(InputStream * const inp):
	AudioFormatReader(inp, formatName),
	flags(0)
	{
		unsigned char buf[1024];
		short leftAudio[1152];	//These two should be unused, but LAME seems to
		short rightAudio[1152];	//require them.
		int retval;

		flags = lame_init(); //Necessary?
		lame_decode_init();

		//Some of this stuff is taken from get_audio.c in the frontend part of
		//lame - I'm not sure how necessary it all is, but the documentation's
		//pretty terrible...
		inp->read(buf, 1024);
		retval = lame_decode1_headers(buf,
									  1024,
									  leftAudio,
									  rightAudio,
									  &headerData);
		if(retval != -1)
		{
			while(headerData.header_parsed != 1)
			{
				inp->read(buf, 1024);
				retval = lame_decode1_headers(buf,
											  1024,
											  leftAudio,
											  rightAudio,
											  &headerData);
				if(retval == -1)
					break;
			}
			if(retval != -1)
			{
				sampleRate = headerData.samplerate;
				bitsPerSample = 16;
				numChannels = headerData.stereo;
				usesFloatingPointData = false;

				if(headerData.nsamp > 0)
					lengthInSamples = headerData.nsamp;
				else
				{
					//Guess from file size and samplerate * bitrate?
				}
			}
		}
	};

	//--------------------------------------------------------------------------
	~LAMEReader()
	{
		lame_decode_exit();
		lame_close(flags);
	};

	//--------------------------------------------------------------------------
	bool read(int **destSamples,
              int64 startSampleInFile,
              int numSamples)
	{
		bool retval = false;

		return retval;
	}
  private:
	///	Header data from the MP3.
	mp3data_struct headerData;
	lame_global_flags *flags;
};

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
LAMEAudioFormat::LAMEAudioFormat():
AudioFormat(formatName, (const tchar**)extensions)
{
	
}

//------------------------------------------------------------------------------
LAMEAudioFormat::~LAMEAudioFormat()
{
	
}

//------------------------------------------------------------------------------
const Array<int> LAMEAudioFormat::getPossibleSampleRates()
{
	//Placeholder.
	const int rates[] = {0};
    return Array <int> (rates);
}

//------------------------------------------------------------------------------
const Array<int> LAMEAudioFormat::getPossibleBitDepths()
{
	//Placeholder.
	const int rates[] = {0};
    return Array <int> (rates);
}

//------------------------------------------------------------------------------
const StringArray LAMEAudioFormat::getQualityOptions()
{
	return StringArray();
}

//------------------------------------------------------------------------------
AudioFormatReader* LAMEAudioFormat::createReaderFor(InputStream *sourceStream,
									                const bool deleteStreamIfOpeningFails)
{
	//Copied from the Ogg AudioFormat.
	LAMEReader* r = new LAMEReader(sourceStream);

    if(r->sampleRate == 0)
    {
        if(!deleteStreamIfOpeningFails)
            r->input = 0;

        deleteAndZero(r);
    }

    return r;
}

//------------------------------------------------------------------------------
AudioFormatWriter* LAMEAudioFormat::createWriterFor(OutputStream *streamToWriteTo,
												    double sampleRateToUse,
												    unsigned int numberOfChannels,
												    int bitsPerSample,
												    const StringPairArray& metadataValues,
												    int qualityOptionIndex)
{
	//Placeholder.
	return 0;
}
