//	LAMEAudioFormat.h - LAME-based audio format for reading MP3s.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef LAMEAUDIOFORMAT_H_
#define LAMEAUDIOFORMAT_H_

#include "juce.h"

///	LAME-based audio format for reading MP3s.
/*!
	It doesn't handle writing to MP3, as I don't need that myself.
 */
class LAMEAudioFormat : public AudioFormat
{
  public:
	///	Constructor.
	LAMEAudioFormat();
	///	Destructor.
	~LAMEAudioFormat();

	///	Returns an array of samplerates the format can handle.
	const Array<int> getPossibleSampleRates();
	///	Returns an array of bitdepths the format can handle.
    const Array<int> getPossibleBitDepths();
	///	Returns whether or not the format can handle stereo.
    bool canDoStereo() {return true;};
	///	Returns whether or not the format can handle mono.
    bool canDoMono() {return true;};
	///	Returns whether or not the format is compressed.
    bool isCompressed() {return true;};
	///	Returns an array of quality options.
    const StringArray getQualityOptions();

	///	Creates an AudioFormatReader for MP3s.
	AudioFormatReader* createReaderFor(InputStream *sourceStream,
                                       const bool deleteStreamIfOpeningFails);
	///	(Doesn't) Create an AudioFormatWriter for MP3s.
    AudioFormatWriter* createWriterFor(OutputStream *streamToWriteTo,
                                       double sampleRateToUse,
                                       unsigned int numberOfChannels,
                                       int bitsPerSample,
                                       const StringPairArray& metadataValues,
                                       int qualityOptionIndex);

    juce_UseDebuggingNewOperator
};

#endif
