//	MP3Tag.cpp - ID3 tag reader using id3lib.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "MP3Tag.h"

#include "id3/tag.h"

using namespace std;

//------------------------------------------------------------------------------
MP3Tag::MP3Tag(const string& filepath):
Tag(filepath)
{
	ID3_Tag file(filepath.c_str());

	//Title.
	{
		ID3_Frame *tempFrame = file.Find(ID3FID_TITLE);
		if(tempFrame)
		{
			ID3_Field *tempField = tempFrame->GetField(ID3FN_TEXT);

			if(tempField)
			{
				if(tempField->GetRawText())
					setTitle(tempField->GetRawText());
			}
		}

		if(getTitle() == "")
			setTitle(getFilename(filepath));
	}

	//Artist.
	{
		ID3_Frame *tempFrame = file.Find(ID3FID_LEADARTIST);
		if(tempFrame)
		{
			ID3_Field *tempField = tempFrame->GetField(ID3FN_TEXT);

			if(tempField)
			{
				if(tempField->GetRawText())
					setArtist(tempField->GetRawText());
			}
		}
	}

	//Album.
	{
		ID3_Frame *tempFrame = file.Find(ID3FID_ALBUM);
		if(tempFrame)
		{
			ID3_Field *tempField = tempFrame->GetField(ID3FN_TEXT);

			if(tempField)
			{
				if(tempField->GetRawText())
					setAlbum(tempField->GetRawText());
			}
		}
	}

	//Track Number.
	{
		ID3_Frame *tempFrame = file.Find(ID3FID_TRACKNUM);
		if(tempFrame)
		{
			ID3_Field *tempField = tempFrame->GetField(ID3FN_TEXT);

			if(tempField)
			{
				if(tempField->GetRawText())
					setTrackNumber(tempField->GetRawText());
			}
		}
	}
}

//------------------------------------------------------------------------------
MP3Tag::~MP3Tag()
{
	
}
