//	OggTag.cpp - Ogg tag reader using Juce's copy of vorbisfile.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "OggTag.h"

#include "src/juce_appframework/audio/audio_file_formats/oggvorbis/vorbisfile.h"

using namespace std;

//------------------------------------------------------------------------------
OggTag::OggTag(const string& filepath):
Tag(filepath)
{
	int i;
	FILE *file;
	OggVorbis_File vorbisFile;
	vorbis_info *fileInfo;
	vorbis_comment *comments;
	unsigned int samplerate;
	unsigned int numSamples;

	file = fopen(filepath.c_str(), "rb");
	if(file)
	{
		if(ov_open(file, &vorbisFile, NULL, 0) > -1)
		{
			if(ov_streams(&vorbisFile) > 0)
			{
				fileInfo = ov_info(&vorbisFile, 0);
				comments = ov_comment(&vorbisFile, -1);

				//Title, Artist, Album, Track Number.
				for(i=0;i<comments->comments;++i)
				{
					string tempstr(comments->user_comments[i]);

					if(tempstr.substr(0, 5) == "TITLE")
						setTitle(tempstr.substr(6));
					else if(tempstr.substr(0, 6) == "ARTIST")
						setArtist(tempstr.substr(7));
					else if(tempstr.substr(0, 5) == "ALBUM")
						setAlbum(tempstr.substr(6));
					else if(tempstr.substr(0, 11) == "TRACKNUMBER")
						setTrackNumber(tempstr.substr(12));
				}
				if(getTitle() == "")
					setTitle(getFilename(filepath));

				//Length.
				samplerate = fileInfo->rate;
				numSamples = ov_pcm_total(&vorbisFile, 0);
				setLength((float)numSamples/(float)samplerate);
			}
			ov_clear(&vorbisFile);
		}
		fclose(file);
	}
}

//------------------------------------------------------------------------------
OggTag::~OggTag()
{
	
}
