//	SongList.h - Visual list of queued songs.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef SONGLIST_H_
#define SONGLIST_H_

#include "Metadata.h"

#include "juce.h"
#include <vector>

class MainWindow;

///	Visual list of queued songs.
class SongList : public Component,
				 public TableListBoxModel
{
  public:
	///	Constructor.
	SongList(MainWindow *win);
	///	Destructor.
	~SongList();

	///	Sizes songs.
	void resized();

	///	Used to load songs into the app.
	bool filesDropped(const StringArray &filenames, int mouseX, int mouseY);
	///	Used to remove all songs from the list.
	void clear();

	///	Returns the number of rows in songs.
	int getNumRows() {return metadata.size();};
	///	Draws the background for songs.
	void paintRowBackground(Graphics& g,
							int rowNumber,
							int width,
							int height,
							bool rowIsSelected);
	///	Draws the contents of a cell.
	void paintCell(Graphics& g,
				   int rowNumber,
				   int columnId,
				   int width,
				   int height,
				   bool rowIsSelected);
	///	Returns the best width for a column.
	int getColumnAutoSizeWidth(int columnId);
	///	Called when the user double-clicks a song.
	void cellDoubleClicked(int rowNumber, int columnId, const MouseEvent& e);
	///	Called when the delete key is pressed.
	void deleteKeyPressed(int lastRowSelected);

	///	Returns the selected rows so they can be dragged to the sandbox.
	const String getDragSourceDescription(const SparseSet<int>& currentlySelectedRows);

	///	Selects the song we're currently playing.
	void setSelectedSong(int val);
	///	Returns the currently-selected song.
	unsigned int getSelectedSong() const;
	///	Returns all the selected songs.
	const SparseSet<int> getSelectedSongs() const;

	///	Returns the selected song's artist.
	const String& getSelectedArtist() const;
	///	Returns the selected song's title.
	const String& getSelectedTitle() const;
	///	Sets the selected song's length.
	void setLength(unsigned int val);

	juce_UseDebuggingNewOperator
  private:
	///	The TableListBox which lists all the songs.
	TableListBox *songs;

	///	Vector of all the songs' metadata.
	std::vector<Metadata> metadata;

	///	A pointer to the main window.
	MainWindow *mainWindow;
};

#endif
