//	Tag.h - Base class for audio file tag readers.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef TAG_H_
#define TAG_H_

#include <string>

///	Base class for audio file tag readers.
class Tag
{
  public:
	  Tag():length(0) {};
	///	Constructor.
	/*!
		Subclasses should open the file and read the relevant tags here.
	 */
	Tag(const std::string& filepath);
	///	Copy Constructor.
	Tag(const Tag& other);
	///	Destructor.
	virtual ~Tag();

	///	Returns the song's title.
	const std::string& getTitle() const {return title;};
	///	Returns the song's Artist.
	const std::string& getArtist() const {return artist;};
	///	Returns the song's Album.
	const std::string& getAlbum() const {return album;};
	///	Returns the song's track number.
	const std::string& getTrackNumber() const {return trackNumber;};
	///	Returns the song's length.
	unsigned int getLength() const {return length;};
  protected:
	///	Sets the song's title.
	void setTitle(const std::string& val);
	///	Sets the song's artist.
	void setArtist(const std::string& val);
	///	Sets the song's album.
	void setAlbum(const std::string& val);
	///	Sets the song's track number.
	void setTrackNumber(const std::string& val);
	///	Sets the song's length.
	void setLength(unsigned int val);

	///	Helper method to return the filename (-path, -extension) of a song.
	const std::string getFilename(const std::string& path);
  private:
	///	The song's title.
	std::string title;
	///	The song's artist.
	std::string artist;
	///	The song's album.
	std::string album;
	///	The song's track number.
	std::string trackNumber;
	///	The song's length.
	unsigned int length;
};

#endif
