//	TagFactory.cpp - Singleton used to generate the correct tag reader based on
//				     file type.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#include "TagFactory.h"
#include "MP3Tag.h"
#include "OggTag.h"
#include "NoMetadataTag.h"
#include "MP3AudioFormat.h"

//------------------------------------------------------------------------------
Tag TagFactory::getMetadata(const String& filepath)
{
	if(filepath.endsWith(T(".mp3")))
		return MP3Tag((const char *)filepath);
	else if(filepath.endsWith(T(".ogg")))
		return OggTag((const char *)filepath);
	else
		return NoMetadataTag((const char *)filepath);
}

//------------------------------------------------------------------------------
AudioFormatReader *TagFactory::getReader(const String& filepath)
{
	if(filepath.endsWith(T(".wav")))
	{
		WavAudioFormat audioFormat;
		FileInputStream *phil = new FileInputStream(File(filepath));

		return audioFormat.createReaderFor(phil, true);
	}
	else if(filepath.endsWith(T(".aif")) ||
			filepath.endsWith(T(".aiff")))
	{
		AiffAudioFormat audioFormat;
		FileInputStream *phil = new FileInputStream(File(filepath));

		return audioFormat.createReaderFor(phil, true);
	}
	else if(filepath.endsWith(T(".flac")))
	{
		FlacAudioFormat audioFormat;
		FileInputStream *phil = new FileInputStream(File(filepath));

		return audioFormat.createReaderFor(phil, true);
	}
	else if(filepath.endsWith(T(".ogg")))
	{
		OggVorbisAudioFormat audioFormat;
		FileInputStream *phil = new FileInputStream(File(filepath));

		return audioFormat.createReaderFor(phil, true);
	}
	else if(filepath.endsWith(T(".mp3")))
	{
		MP3AudioFormat audioFormat;
		FileInputStream *phil = new FileInputStream(File(filepath));

		return audioFormat.createReaderFor(phil, true);
	}
	else
		return 0;
}
