//	app.h - Main application stuff.
//	----------------------------------------------------------------------------
//	This file is part of NiallTunes, a basic music player.
//	Copyright (c) 2007 Niall Moody.
//
//	This program is free software: you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation, either version 3 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program.  If not, see <http://www.gnu.org/licenses/>.
//	----------------------------------------------------------------------------

#ifndef APP_H_
#define APP_H_

#include "juce.h"

class MainWindow;

//------------------------------------------------------------------------------
///	Handles the main application stuff.
class App : public JUCEApplication
{
  public:
	///	Constructor.
	App() {win = 0;};
	///	Destructor.
	~App() {};

	///	Where all the important stuff happens for this class.
	void initialise(const String& commandLine);
	///	Not used by us?
	void shutdown();

	///	Returns the name of the application (duh...).
	const String getApplicationName() {return T("NiallTunes");};
	///	Returns the current version.
	const String getApplicationVersion() {return T("0.10");};
	///	Whether or not we want to allow more than one instance to run.
	bool moreThanOneInstanceAllowed() {return true;};
	///	Not used right now?
	void anotherInstanceStarted(const String& commandLine) {};

	///	Returns the main window.
	DocumentWindow *getWindow() {return win;};
  private:
	///	Pointer to our main window.
	DocumentWindow *win;
};

//------------------------------------------------------------------------------
///	Pointless waste of code, just to receive messages from the close button.
class StupidWindow : public DocumentWindow
{
  public:
	///	Constructor.
	StupidWindow(const String& commandLine);
	///	Constructor.
	~StupidWindow();

	///	To handle the close button being pressed.
	void closeButtonPressed();
  private:
	///	The main window part.
	MainWindow *mainWindow;
};

#endif
